import time
import pandas as pd
from zaifbot.moving_average import get_ema


def get_macd(currency_pair, period='1d', count=5, short=12, long=26, signal=9, to_epoch_time=int(time.time())):
    # 短期、長期のEMAを取得
    short_ema = get_ema(currency_pair, period, count, to_epoch_time, short)
    long_ema = get_ema(currency_pair, period, count, to_epoch_time, long)

    df_short = pd.DataFrame(short_ema['return']['ema'])
    df_long = pd.DataFrame(long_ema['return']['ema'])
    df_short.rename(columns={'moving_average': 'short_ema'}, inplace=True)
    df_long.rename(columns={'moving_average': 'long_ema'}, inplace=True)

    df = pd.merge(df_short, df_long)
    df = df[df['short_ema'] != 0.0]

    # macd = 短期EMA－長期EMA
    df['macd'] = df['short_ema'] - df['long_ema']

    # signalはmacdのEMA
    df['signal'] = df['macd'].ewm(span=signal).mean()
    df['macd2'] = df['macd'] - df['signal']
    df.drop(['close', 'short_ema', 'long_ema', 'closed'], axis=1, inplace=True)

    return {'success': 1, 'return': {'macds': df.to_dict(orient='records')}}



from zaifbot.modules.api.stream import ZaifLastPrice
from zaifbot.modules.api.cache import ZaifCurrencyPairs


def get_current_last_price(currency_pair):
    api = ZaifLastPrice()
    return api.last_price(currency_pair)


def get_more_executable_price(currency_pair, price, *, is_buy):
    currency_pair_info = _get_currency_pair_info(currency_pair)
    if is_buy:
        return price + (currency_pair_info['aux_unit_step'] -
                        (price % currency_pair_info['aux_unit_step']))
    else:
        return price - (price % currency_pair_info['aux_unit_step'])


def get_buyable_amount(currency_pair, amount, price):
    buyable_amount = amount / price
    return get_round_amount(currency_pair, buyable_amount)


def get_round_amount(currency_pair, amount):
    currency_pair_info = _get_currency_pair_info(currency_pair)
    return amount - (amount % currency_pair_info['item_unit_step'])


def _get_currency_pair_info(currency_pair):
    currency_pair_infos = ZaifCurrencyPairs()
    return currency_pair_infos[currency_pair]



from sqlalchemy import create_engine, MetaData
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker


def get_connection_string():
    # マルチデータベースを実現したい場合、ここを修正する
    return 'sqlite:///{}'.format('zaif_bot.db')


Base = declarative_base()
engine = create_engine(get_connection_string())
metadata = MetaData(engine)
Session = sessionmaker(bind=engine)


def get_session():
    return Session()



import numpy as np
from zaifbot.bot_common.bot_const import PERIOD_SECS, LIMIT_COUNT
from zaifbot.bot_common.logger import logger
from zaifbot.models.moving_average import TradeLogs, MovingAverages
from zaifbot.modules.api.wrapper import BotPublicApi
from zaifbot.modules.dao.moving_average import TradeLogsDao, MovingAverageDao


def get_need_epoch_times(start_time, end_time, period):
    while True:
        yield start_time
        start_time += PERIOD_SECS[period]
        if start_time >= end_time:
            yield end_time
            break


class TradeLogsSetUp:
    def __init__(self, currency_pair, period, count, length):
        self._currency_pair = currency_pair
        self._period = period
        self._count = count
        self._length = length
        self._trade_logs = TradeLogsDao(self._currency_pair, self._period)

    def execute(self, start_time, end_time):
        target_epoch_times = self._get_target_epoch_times(start_time, end_time)
        if len(target_epoch_times) == 0:
            return True
        api_records = self._get_ohlc_data_from_server(end_time)
        if len(api_records) == 0:
            return False
        target_trade_logs_record =\
            list(filter(lambda x: x['time'] in target_epoch_times, api_records))
        trade_logs_model_data = self._set_trade_logs_model_data(target_trade_logs_record)
        return self._trade_logs.create_data(trade_logs_model_data)

    def _get_target_epoch_times(self, start_time, end_time):
        trade_logs_record = self._trade_logs.get_records(end_time, start_time, True)
        return self._check_missing_records(trade_logs_record, start_time, end_time, self._period)

    @staticmethod
    def _check_missing_records(trade_logs_record, start_time, end_time, period):
        to_epoch_times = set([x.time for x in trade_logs_record])
        target_epoch_times = set()
        for need_epoch_time in get_need_epoch_times(start_time, end_time, period):
            if need_epoch_time not in to_epoch_times:
                target_epoch_times.add(need_epoch_time)
        return target_epoch_times

    def _set_trade_logs_model_data(self, target_trade_logs_record):
        trade_logs_model_data = []
        for trade_logs in target_trade_logs_record:
            trade_logs_model_data.append(TradeLogs(
                time=trade_logs['time'],
                currency_pair=self._currency_pair,
                period=self._period,
                open=trade_logs['open'],
                high=trade_logs['high'],
                low=trade_logs['low'],
                close=trade_logs['close'],
                average=trade_logs['average'],
                volume=trade_logs['volume'],
                closed=int(trade_logs['closed'])
            ))
        return trade_logs_model_data

    def _get_ohlc_data_from_server(self, end_time):
        public_api = BotPublicApi()
        api_params = {'period': self._period, 'count': LIMIT_COUNT, 'to_epoch_time': end_time + 1}
        try:
            api_record = public_api.everything('ohlc_data', self._currency_pair, api_params)
        except Exception as e:
            logger.error(e)
            api_record = []
        required_count = self._count + self._length
        if required_count <= LIMIT_COUNT:
            return api_record
        count = required_count - LIMIT_COUNT
        second_end_time = end_time - (LIMIT_COUNT * PERIOD_SECS[self._period]) + 1
        second_api_params =\
            {'period': self._period, 'count': count, 'to_epoch_time': second_end_time}
        try:
            second_api_record =\
                public_api.everything('ohlc_data', self._currency_pair, second_api_params)
        except Exception as e:
            logger.error(e)
            second_api_record = []
        api_record = second_api_record + api_record
        return api_record


class MovingAverageSetUp:
    def __init__(self, currency_pair, period, count, length):
        self._currency_pair = currency_pair
        self._period = period
        self._count = count
        self._length = length
        self._moving_average = MovingAverageDao(self._currency_pair, self._period, self._length)

    def execute(self, ma_start_time, tl_start_time, end_time):
        moving_average_model_data = set()
        target_epoch_times = self._get_target_epoch_times(ma_start_time, end_time)
        if len(target_epoch_times) == 0:
            return True
        trade_logs_moving_average =\
            self._moving_average.get_trade_logs_moving_average(end_time, tl_start_time)
        for i in self._get_moving_average(trade_logs_moving_average, target_epoch_times):
            moving_average_model_data.add(self._get_moving_average_model_dataset(i['time'],
                                                                                 i['sma'],
                                                                                 i['ema'],
                                                                                 i['closed']))
        return self._moving_average.create_data(moving_average_model_data)

    def _get_target_epoch_times(self, start_time, end_time):
        moving_average_record = self._moving_average.get_records(end_time, start_time, True)
        return self._check_missing_records(moving_average_record,
                                           start_time,
                                           end_time,
                                           self._period)

    @staticmethod
    def _check_missing_records(moving_average_record, start_time, end_time, period):
        to_epoch_times = set([x.time for x in moving_average_record])
        target_epoch_times = set()
        for need_epoch_time in get_need_epoch_times(start_time, end_time, period):
            if need_epoch_time not in to_epoch_times:
                target_epoch_times.add(need_epoch_time)
        return target_epoch_times

    def _get_moving_average(self, trade_logs_moving_average, target_epoch_times):
        moving_average = {'time': 0}
        for i in range(self._length, len(trade_logs_moving_average)):
            if trade_logs_moving_average[i].TradeLogs.time in target_epoch_times:
                nums = self._get_nums(trade_logs_moving_average, i)
                sma = self._get_sma(nums)
                ema = self._get_ema(trade_logs_moving_average[i - 1], nums, moving_average)
                moving_average = {'time': trade_logs_moving_average[i].TradeLogs.time,
                                  'sma': sma,
                                  'ema': ema,
                                  'closed': trade_logs_moving_average[i].TradeLogs.closed}
                yield moving_average

    def _get_nums(self, trade_logs_moving_average, i):
        nums = []
        for j in range(0, self._length + 1):
            nums.append(trade_logs_moving_average[i - j].TradeLogs.close)
        return nums

    def _get_ema(self, last_trade_logs_moving_average, nums, moving_average):
        current_price = nums.pop(0)
        if last_trade_logs_moving_average.MovingAverages is not None\
                and last_trade_logs_moving_average.MovingAverages.closed:
            last_ema = last_trade_logs_moving_average.MovingAverages.ema
        elif last_trade_logs_moving_average.TradeLogs.time == moving_average['time']:
            last_ema = moving_average['ema']
        else:
            last_ema = np.average(nums)
        k = 2 / (self._length + 1)
        return current_price * k + last_ema * (1 - k)

    @staticmethod
    def _get_sma(nums):
        return np.average(nums[0:-1])

    def _get_moving_average_model_dataset(self, time, sma, ema, closed):
        return MovingAverages(
            time=time,
            currency_pair=self._currency_pair,
            period=self._period,
            length=self._length,
            sma=sma,
            ema=ema,
            closed=closed)



from sqlalchemy import Column, Integer, Float, String

from zaifbot.models import Base


class TradeLogs(Base):
    __tablename__ = 'trade_logs'
    time = Column('time', Integer, primary_key=True)
    currency_pair = Column('currency_pair', String, primary_key=True)
    period = Column('period', String, primary_key=True)
    open = Column('open', Float, nullable=False)
    high = Column('high', Float, nullable=False)
    low = Column('low', Float, nullable=False)
    close = Column('close', Float, nullable=False)
    average = Column('average', Float, nullable=False)
    volume = Column('volume', Float, nullable=False)
    closed = Column('closed', Integer, nullable=False)


class MovingAverages(Base):
    __tablename__ = 'moving_averages'
    time = Column('time', Integer, primary_key=True)
    currency_pair = Column('currency_pair', String, primary_key=True)
    period = Column('period', String, primary_key=True)
    length = Column('length', Integer, primary_key=True)
    sma = Column('sma', Float, nullable=False)
    ema = Column('ema', Float, nullable=False)
    closed = Column('closed', Integer, nullable=False)





class ZaifBot:
    _process = []

    def add_running_process(self, auto_trade_process):
        self._process.append(auto_trade_process)

    def start(self):
        running_processes = []
        for process in self._process:
            process.start()
            running_processes.append(process)
        [x.join() for x in running_processes]



from sqlalchemy import and_
from zaifbot.modules.dao import DaoBase
from zaifbot.models.bollinger_bands import BollingerBands
from sqlalchemy import exc
from zaifbot.bot_common.bot_const import CLOSED


class BollingerBandsDao(DaoBase):

    def __init__(self, currency_pair, period, length):
        super().__init__()
        self._currency_pair = currency_pair
        self._period = period
        self._length = length

    def get_model(self):
        return BollingerBands

    def get_records(self, end_time, start_time, closed):
        session = self.get_session()
        select_query = session.query(self.model)
        if closed:
            select_query = select_query.filter(and_(self.model.time <= end_time,
                                                    self.model.time > start_time,
                                                    self.model.currency_pair == self._currency_pair,
                                                    self.model.period == self._period,
                                                    self.model.length == self._length,
                                                    self.model.closed == CLOSED
                                                    ))
        else:
            select_query = select_query.filter(and_(self.model.time <= end_time,
                                                    self.model.time > start_time,
                                                    self.model.currency_pair == self._currency_pair,
                                                    self.model.period == self._period,
                                                    self.model.length == self._length
                                                    ))
        result = select_query.order_by(self.model.time).all()
        session.close()
        return result

    def create_data(self, bollinger_bands):
        session = self.get_session()
        try:
            for record in bollinger_bands:
                session.merge(record)
            session.commit()
            session.close()
            return True
        except exc.SQLAlchemyError:
            session.rollback()
            session.close()
        return False



from zaifbot.modules.dao.bollinger_bands import BollingerBandsDao
from zaifbot.modules.moving_average import get_need_epoch_times
from zaifbot.models.bollinger_bands import BollingerBands
from zaifbot.moving_average import get_sma
import numpy as np


class BollingerBandsSetUp:
    def __init__(self, currency_pair, period, count, length):
        self._currency_pair = currency_pair
        self._period = period
        self._count = count
        self._length = length
        self._bollinger_bands = BollingerBandsDao(self._currency_pair, self._period, self._length)

    def execute(self, start_time, end_time):
        bollinger_bands_model_data = set()
        target_epoch_times = self._get_target_epoch_times(start_time, end_time)
        if len(target_epoch_times) == 0:
            return True
        sma_records = get_sma(self._currency_pair, self._period,
                              self._count, end_time, self._length)
        if sma_records['success'] == 0:
            return False
        for i in range(self._length, len(sma_records['return']['sma'])):
            if sma_records['return']['sma'][i]['time_stamp'] in target_epoch_times:
                nums = self._get_nums(sma_records['return']['sma'], i)
                standard_deviation = self._get_standard_deviation(nums)
                bollinger_bands_model_data.add(self._get_bollinger_bands_model_dataset
                                               (sma_records['return']['sma'][i]['time_stamp'],
                                                sma_records['return']['sma'][i]['moving_average'],
                                                standard_deviation,
                                                sma_records['return']['sma'][i]['closed']))
        return self._bollinger_bands.create_data(bollinger_bands_model_data)

    def _get_target_epoch_times(self, start_time, end_time):
        bollinger_bands_record = self._bollinger_bands.get_records(end_time, start_time, True)
        return self._check_missing_records(bollinger_bands_record, start_time,
                                           end_time, self._period)

    @staticmethod
    def _check_missing_records(bollinger_bands_record, start_time, end_time, period):
        exist_epoch_times = set([x.time for x in bollinger_bands_record])
        target_epoch_times = set()
        for need_epoch_time in get_need_epoch_times(start_time, end_time, period):
            if need_epoch_time not in exist_epoch_times:
                target_epoch_times.add(need_epoch_time)
        return target_epoch_times

    @staticmethod
    def _get_standard_deviation(nums):
        return np.sqrt(np.average(np.power(nums, 2)))

    def _get_nums(self, sma_records, i):
        nums = []
        for j in range(0, self._length):
            nums.append(sma_records[i - j]['close'] - sma_records[i]['moving_average'])
        return nums

    def _get_bollinger_bands_model_dataset(self, time, sma, standard_deviation, closed):
        return BollingerBands(
            time=time,
            currency_pair=self._currency_pair,
            period=self._period,
            length=self._length,
            sd1p=sma + standard_deviation,
            sd2p=sma + (standard_deviation * 2),
            sd3p=sma + (standard_deviation * 3),
            sd1n=sma - standard_deviation,
            sd2n=sma - (standard_deviation * 2),
            sd3n=sma - (standard_deviation * 3),
            closed=closed)



import time
from abc import ABCMeta, abstractmethod
from datetime import datetime
from threading import Thread, Event
from uuid import uuid4
from zaifbot.bot_common.errors import ZaifBotError
from zaifbot.bot_common.logger import logger
from zaifbot.modules.api.cache import ZaifCurrencyPairs
from zaifbot.modules.utils import get_current_last_price


class AutoCancelClient:
    def __init__(self, trade_api):
        self._auto_cancel_orders = {}
        self._trade_api = trade_api

    def cancel_by_time(self, order_id, currency_pair, wait_sec):
        auto_cancel = _AutoCancelByTime(self._trade_api, order_id, currency_pair, wait_sec)
        auto_cancel.start()
        self._auto_cancel_orders[auto_cancel.id] = auto_cancel
        return auto_cancel.get_info()

    def cancel_by_price(self, order_id, currency_pair, target_margin):
        auto_cancel = _AutoCancelByPrice(self._trade_api, order_id, currency_pair, target_margin)
        auto_cancel.start()
        self._auto_cancel_orders[auto_cancel.id] = auto_cancel
        return auto_cancel.get_info()

    def get_active_cancel_orders(self):
        self._remove_dead_threads()
        active_cancel_orders = []
        for auto_cancel_order in self._auto_cancel_orders.values():
            active_cancel_orders.append(auto_cancel_order.get_info())
        return active_cancel_orders

    def stop_cancel(self, cancel_id):
        self._remove_dead_threads()
        cancel_order = self._auto_cancel_orders.get(cancel_id, None)
        if cancel_order is None:
            logger.warn('couldn\'t find cancel_id you gave : {}'.format(cancel_id))
            return
        cancel_order.stop()
        cancel_order.join()
        logger.info('cancel stopped: {{ {} }}'.format(cancel_order.get_info()))
        self._remove_dead_threads()
        return cancel_order.get_info()

    def _remove_dead_threads(self):
        delete_cancel_ids = []
        for cancel_id, cancel_thread in self._auto_cancel_orders.items():
            if cancel_thread.is_alive() is False:
                delete_cancel_ids.append(cancel_id)
        for cancel_id in delete_cancel_ids:
            del self._auto_cancel_orders[cancel_id]


_SLEEP_TIME = 1


class _AutoCancelOrder(Thread, metaclass=ABCMeta):
    def __init__(self, trade_api, order_id, currency_pair):
        super().__init__(daemon=True)
        self._private_api = trade_api
        self._id = str(uuid4())
        self._order_id = order_id
        self._currency_pair = currency_pair
        self._is_token = self._is_token(currency_pair)
        self._stop_event = Event()
        self._start_time = None

    def run(self):
        self._start_time = time.time()
        while self._stop_event.is_set() is False:
            active_orders = self._private_api.active_orders(currency_pair=self._currency_pair)
            if str(self._order_id) not in active_orders:
                self.stop()
                continue
            if self._need_cancel_now():
                self._execute()
            else:
                time.sleep(_SLEEP_TIME)

    def get_info(self):
        info = {
            'id': self.id,
            'cancel_type': self.get_type(),
            'order_id': self._order_id,
            'currency_pair': self._currency_pair,
            'is_token': self._is_token,
            'current_price': get_current_last_price(self._currency_pair)['last_price'],
            'auto_cancel_started': self._start_time,
        }
        return info

    def _execute(self):
        self._private_api.cancel_order(order_id=self._order_id, is_token=self._is_token)
        logger.info('order canceled \n {{order_id: {0}, timestamp: {1}}}'.format(self._order_id, datetime.now()))
        self.stop()

    def stop(self):
        self._stop_event.set()

    @staticmethod
    def _is_token(currency_pair):
        currency_pairs = ZaifCurrencyPairs()
        record = currency_pairs[currency_pair]
        if record:
            return record['is_token']
        raise ZaifBotError('illegal currency_pair:{}'.format(currency_pair))

    @property
    def id(self):
        return self._id

    @abstractmethod
    def get_type(self):
        raise NotImplementedError

    @abstractmethod
    def _need_cancel_now(self):
        raise NotImplementedError


class _AutoCancelByTime(_AutoCancelOrder):
    def __init__(self, trade_api, order_id, currency_pair, wait_sec):
        super().__init__(trade_api, order_id, currency_pair)
        self._wait_sec = wait_sec

    def get_info(self):
        info = super().get_info()
        info['wait_sec'] = self._wait_sec
        info['rest_time'] = self._wait_sec - (time.time() - self._start_time)
        return info

    def _need_cancel_now(self):
        if time.time() - self._start_time < self._wait_sec:
            return False
        return True

    def get_type(self):
        return 'by_time'


class _AutoCancelByPrice(_AutoCancelOrder):
    def __init__(self, trade_api, order_id, currency_pair, target_margin):
        super().__init__(trade_api, order_id, currency_pair)
        self._target_margin = target_margin
        self._currency_pair = currency_pair
        self._start_price = None

    def get_info(self):
        info = super().get_info()
        info['start_price'] = self._start_price
        info['target_margin'] = self._target_margin
        info['current_margin'] = abs(info['current_price'] - self._start_price)
        return info

    def run(self):
        self._start_price = get_current_last_price(self._currency_pair)['last_price']
        super().run()

    def _need_cancel_now(self):
        last_price = get_current_last_price(self._currency_pair)['last_price']
        if abs(self._start_price - last_price) < self._target_margin:
            return False
        return True

    def get_type(self):
        return 'by_price'



import time
import pandas as pd
from pandas import DataFrame, Series
from zaifbot.modules.api.wrapper import BotPublicApi


def get_adx(currency_pair, period='1d', count=5, length=14, to_epoch_time=int(time.time())):
    public_api = BotPublicApi()
    second_api_params = {'period': period, 'count': count + 2 * length - 2, 'to_epoch_time': to_epoch_time}
    price_infos = DataFrame(public_api.everything('ohlc_data', currency_pair, second_api_params))
    return _get_adx(price_infos[['high', 'low', 'time', 'close']].copy(), length, count)


def _get_adx(df, length, count):
    def _create_dict(df, count):
        adxs = df.ix[len(df) - count:, ['time', 'ADX']].to_dict(orient='records')
        return {'success': 1, 'return': {'adxs': adxs}}

    df = pd.concat([df, _get_dm(df)], axis=1)
    df['TR'] = _get_tr(df)
    df = _get_di(df, length, count)
    df['DX'] = abs(df['+DI'] - df['-DI']) / (df['+DI'] + df['-DI']) * 100
    df['ADX'] = (df['DX']).rolling(window=length).mean()
    return _create_dict(df, count)


def _get_dm(df):
    """引数にはhigh,low, closeをカラム名に持つDataFrameをとる"""
    def _calc_dm(row):
        high_dff = max(row['high'], 0)
        low_diff = min(row['low'], 0)

        if abs(high_dff) > abs(low_diff):
            return Series({'+DM': high_dff, '-DM': 0})
        elif abs(high_dff) < abs(low_diff):
            return Series({'+DM': 0, '-DM': abs(low_diff)})
        else:
            return Series({'+DM': 0, '-DM': 0})

    return df.diff().fillna(0).apply(_calc_dm, axis=1)


def _get_tr(df):
    """引数にはhigh, low, closeをカラム名に持つDataFrameをとる"""
    new_row = [0]
    for i in range(1, len(df)):
        tdy_high_minus_ystday_high = df.ix[i, 'high'] - df.ix[i, 'low']
        tdy_high_minus_ystday_close = df.ix[i, 'high'] - df.ix[i - 1, 'close']
        ystday_close_minus_tdy_low = df.ix[i - 1, 'close'] - df.ix[i, 'low']
        new_row.append(max(tdy_high_minus_ystday_high, tdy_high_minus_ystday_close, ystday_close_minus_tdy_low))
    return new_row


def _get_di(df, length, count):
    for i in range(count + length -1):
        df.ix[i + length - 1, '+DI'] = (df.ix[i:i + length - 1, '+DM'].sum() / df.ix[i:i + length - 1, 'TR'].sum()) * 100
        df.ix[i + length - 1, '-DI'] = (df.ix[i:i + length - 1, '-DM'].sum() / df.ix[i:i + length - 1, 'TR'].sum()) * 100
    return df.fillna(0)



from abc import ABCMeta, abstractmethod
from zaifbot.models import Base, engine, get_session
from zaifbot.models import moving_average, bollinger_bands


class DaoBase(metaclass=ABCMeta):
    def __init__(self):
        self.model = self.get_model()

    @classmethod
    def get_session(cls):
        return get_session()

    @abstractmethod
    def get_model(self):
        raise NotImplementedError()


def init_database():
    Base.metadata.create_all(bind=engine)



import traceback
from datetime import datetime, timedelta
from threading import Thread, Event, Lock
from zaifapi.impl import ZaifPublicStreamApi
from zaifbot.bot_common.errors import ZaifBotError
from zaifbot.bot_common.logger import logger
from zaifbot.modules.api.wrapper import BotPublicApi
from .cache import ZaifCurrencyPairs


class ZaifLastPrice:
    _instance = None
    _lock = Lock()
    _threads = {}
    _stop_events = {}
    _last_prices = {}
    _currency_pairs = None

    def __new__(cls):
        with cls._lock:
            if cls._instance is None:
                cls._instance = super().__new__(cls)
        return cls._instance

    def _get_target_thread(self, currency_pair):
        def create_stream_thread():
            stop_event = Event()
            error_event = Event()
            thread_obj = _StreamThread(currency_pair, stop_event, error_event)
            thread_obj.start()
            self._threads[currency_pair] = thread_obj
            self._stop_events[currency_pair] = stop_event
        if currency_pair not in self._threads:
            create_stream_thread()
        if self._threads[currency_pair].is_error_happened:
            create_stream_thread()
        if self._threads[currency_pair].is_alive() is False:
            raise ZaifBotError('thread is dead')
        return self._threads[currency_pair]

    def last_price(self, currency_pair):
        def get_token_last_price():
            api = BotPublicApi()
            last_price = api.last_price(currency_pair)['last_price']
            jst_time = datetime.utcnow() + timedelta(hours=9)
            jst_time_str = jst_time.strftime('%Y-%m-%d %H:%M:%S.%f')
            return {'timestamp': jst_time_str, 'last_price': last_price}

        def is_token():
            currency_pairs = ZaifCurrencyPairs()
            currency_pair_rec = currency_pairs[currency_pair]
            if currency_pair_rec:
                return currency_pair_rec['is_token']
            raise ZaifBotError('illegal currency pair:{}'.format(currency_pair))
        if is_token():
            return get_token_last_price()
        receive = self._get_target_thread(currency_pair).last_receive
        return {'timestamp': receive['timestamp'], 'last_price': receive['last_price']['price']}

    def close_all_socket(self):
        [event.set() for event in self._stop_events.values()]
        [thread.join() for thread in self._threads.values()]


class _StreamThread(Thread):

    def __init__(self, currency_pair, stop_event, error_event):
        super(_StreamThread, self).__init__(name='{}_wss_stream'.format(currency_pair), daemon=True)
        self._currency_pair = currency_pair
        self._stop_event = stop_event
        self._last_receive = None
        self._stream_api = ZaifPublicStreamApi()
        self._set_first_last_price()
        self._error_event = error_event

    def run(self):
        try:
            for receive in self._stream_api.execute(self._currency_pair):
                self._last_receive = receive
                if self._stop_event.is_set():
                    self._stream_api.stop()
        except Exception as e:
            logger.error(str(e))
            logger.error(traceback.format_exc())
            self._error_event.set()

    def _set_first_last_price(self):
        self._last_receive = next(self._stream_api.execute(self._currency_pair))

    @property
    def last_receive(self):
        return self._last_receive

    @property
    def is_error_happened(self):
        return self._error_event.is_set()



import traceback
import time
import random
from zaifbot.bot_common.logger import logger
from zaifapi.impl import ZaifTradeApi, ZaifPublicApi
from zaifapi.api_error import ZaifApiNonceError, ZaifApiError

_RETRY_COUNT = 5
_WAIT_SECOND = 5


def _with_retry(func):
    def _wrapper(self, *args, **kwargs):
        for i in range(_RETRY_COUNT):
            try:
                return func(self, *args, **kwargs)
            except ZaifApiError as e:
                logger.error(e)
                logger.error(traceback.format_exc())
                raise e
            except ZaifApiNonceError as e:
                logger.error(e)
                logger.error(traceback.format_exc())
                a = random.uniform(0.5, 1.0)
                time.sleep(a)
                continue
            except Exception as e:
                logger.error(e)
                logger.error(traceback.format_exc())
                time.sleep(_WAIT_SECOND)
                continue
    return _wrapper


class BotTradeApi(ZaifTradeApi):
    def __init__(self, key, secret):
        super().__init__(key, secret)

    @_with_retry
    def active_orders(self, **kwargs):
        return super().active_orders(**kwargs)

    @_with_retry
    def cancel_order(self, **kwargs):
        return super().cancel_order(**kwargs)

    @_with_retry
    def deposit_history(self, **kwargs):
        return super().deposit_history(**kwargs)

    @_with_retry
    def get_id_info(self):
        return super().get_id_info()

    @_with_retry
    def get_info(self):
        return super().get_info()

    @_with_retry
    def get_info2(self):
        return super().get_info2()

    @_with_retry
    def get_personal_info(self):
        return super().get_personal_info()

    @_with_retry
    def trade(self, **kwargs):
        return super().trade(**kwargs)

    @_with_retry
    def trade_history(self, **kwargs):
        return super().trade_history(**kwargs)

    @_with_retry
    def withdraw(self, **kwargs):
        return super().withdraw(**kwargs)

    @_with_retry
    def withdraw_history(self, **kwargs):
        return super().withdraw_history(**kwargs)


class BotPublicApi(ZaifPublicApi):

    @_with_retry
    def last_price(self, currency_pair):
        return super().last_price(currency_pair)

    @_with_retry
    def ticker(self, currency_pair):
        return super().ticker(currency_pair)

    @_with_retry
    def trades(self, currency_pair):
        return super().trades(currency_pair)

    @_with_retry
    def depth(self, currency_pair):
        return super().depth(currency_pair)

    @_with_retry
    def currency_pairs(self, currency_pair):
        return super().currency_pairs(currency_pair)

    @_with_retry
    def currencies(self, currency):
        return super().currencies(currency)

    @_with_retry
    def everything(self, func_name, currency_pair, params):
        return super().everything(func_name, currency_pair, params)



from threading import Lock
from zaifbot.modules.api.wrapper import BotPublicApi


class ZaifCurrencyPairs:
    _instance = None
    _lock = Lock()
    _threads = {}
    _stop_events = {}
    _last_prices = {}
    _currency_pairs = None

    def __new__(cls):
        with cls._lock:
            if cls._instance is None:
                cls._instance = super().__new__(cls)
                api = BotPublicApi()
                cls._currency_pairs = api.currency_pairs('all')
        return cls._instance

    def __getitem__(self, currency_pair):
        record = list(filter(lambda x: x['currency_pair'] == currency_pair, self._currency_pairs))
        if record:
            return record[0]
        return None



import time
from zaifbot.moving_average import get_end_time
from zaifbot.modules.bollinger_bands import BollingerBandsSetUp
from zaifbot.bot_common.bot_const import PERIOD_SECS, LIMIT_COUNT, LIMIT_LENGTH
from zaifbot.modules.dao.bollinger_bands import BollingerBandsDao


def get_bollinger_bands(currency_pair='btc_jpy', period='1d', count=LIMIT_COUNT,
                        to_epoch_time=int(time.time()), length=LIMIT_LENGTH):
    count = min(count, LIMIT_COUNT)
    length = min(length, LIMIT_LENGTH)
    end_time = get_end_time(to_epoch_time, period)
    start_time = end_time - (count * PERIOD_SECS[period])
    bollinger_bands = BollingerBandsSetUp(currency_pair, period, count, length)

    if bollinger_bands.execute(start_time, end_time) is False:
        return {'success': 0, 'error': 'failed to set up bollinger bands'}
    return _create_return_dict(currency_pair, period, length, end_time, start_time, count)


def _create_return_dict(currency_pair, period, length, end_time, start_time, count):
    return_datas = []
    bollinger_bands = BollingerBandsDao(currency_pair, period, length)
    bollinger_bands_result = bollinger_bands.get_records(end_time, start_time, False)
    if len(bollinger_bands_result) < count:
        return {'success': 0, 'error': 'bollinger bands data is missing'}
    for i in bollinger_bands_result:
        return_datas.append({
                            'time_stamp': i.time,
                            'sd1p': i.sd1p, 'sd2p': i.sd2p, 'sd3p': i.sd3p,
                            'sd1n': i.sd1n, 'sd2n': i.sd2n, 'sd3n': i.sd3n})
    return {'success': 1, 'return': {'bollinger_bands': return_datas}}



from zaifbot.modules.dao import DaoBase
# todo: AutoTradeが存在しない
from zaifbot.models.auto_trade import AutoTrade
from sqlalchemy import exc


class AutoTradeDao(DaoBase):

    def __init__(self, start_time):
        super().__init__()
        self._start_time = start_time

    def get_model(self):
        return AutoTrade

    def get_record(self, start_time):
        session = self.get_session()
        result = session.query(self.model).filter(self.model.start_time == start_time).all()
        session.close()
        return result

    def create_data(self, auto_trade_data):
        session = self.get_session()
        try:
            session.merge(auto_trade_data)
            session.commit()
            session.close()
            return True
        except exc.SQLAlchemyError:
            session.rollback()
            session.close()
        return False



from setuptools import setup, find_packages

setup(
    name='zaifbot',
    version='0.0.1',
    description='Zaif Bot Library',
    long_description='https://pypi.python.org/pypi/zaifbot',
    url='https://github.com/Akira-Taniguchi/zaifbot',
    author='AkiraTaniguchi Monji',
    author_email='a.taniguchi@techbureau.jp ttskmnj@gmail.com',
    packages=find_packages(),
    license='MIT',
    keywords='zaif bit coin btc xem mona jpy virtual currency block chain bot',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License'
    ],
    install_requires=[
        'zaifapi==1.5.2',
        'numpy==1.12.0',
        'pandas==0.20.1',
        'matplotlib==2.0.2',
        'reportlab==3.4.0',
        'SQLAlchemy==1.1.5',
        'websocket-client==0.40.0',
        'slackclient==1.0.5'
    ],
    entry_points="""\
      [console_scripts]
      update_assignee_id = zaifbot:main
      init_database = zaifbot.modules.dao:init_database
      """,
    include_package_data=True
)



from slackclient import SlackClient


class _SlackNotifier:
    def __init__(self, slack_token):
        self._slack_client = SlackClient(slack_token)

    def send_message(self, channel_id, message, username):
        return self._slack_client.api_call(
            "chat.postMessage",
            channel=channel_id,
            text=message,
            username=username
        )


def send_slack_message(slack_token, channel_id, message, username):
    slack_notifier = _SlackNotifier(slack_token)
    return slack_notifier.send_message(channel_id, message, username)



import time
from pandas import DataFrame
from zaifbot.modules.api.wrapper import BotPublicApi


def get_rsi(currency_pair, period='1d', count=5, length=14, to_epoch_time=int(time.time())):
    public_api = BotPublicApi()
    second_api_params = {'period': period, 'count': count + length, 'to_epoch_time': to_epoch_time}
    price_infos = DataFrame(public_api.everything('ohlc_data', currency_pair, second_api_params))
    return _get_rsi(price_infos[['close', 'time']], count, length)


def _get_rsi(price_infos, count, length):
    def _calc_first_a_and_b(close):
        decreased_sum, increased_sum, length = 0, 0, len(close)
        for i in range(1, length):
            decreased_sum += abs(min(close.ix[i] - close.ix[i - 1], 0))
            increased_sum += max(close.ix[i] - close.ix[i - 1], 0)
        return increased_sum / length, decreased_sum / length

    def _generate_next_ab(a, b, length):
        a, b, = a, b
        while True:
            diff = yield a, b
            a = _calc_next_ab(a, max(diff, 0), length)
            b = _calc_next_ab(b, abs(min(diff, 0)), length)

    def _rsi_formula(a, b):
        return a / (a + b) * 100

    def _calc_next_ab(before, diff, length):
        return (before * (length - 1) + diff) / length

    def _create_dict(t, rsi):
        return {'timestamp': t, 'rsi': rsi}

    results = []
    closes = price_infos['close']
    times = price_infos['time']

    # 1本目のRSIの計算
    a_1, b_1 = _calc_first_a_and_b(closes.ix[:length])
    t_1 = times.ix[length]
    results.append(_create_dict(t_1, _rsi_formula(a_1, b_1)))

    # 2本目以降のRSIの計算
    generator = _generate_next_ab(a_1, b_1, length)
    next(generator)
    for i in range(count - 1):
        diff = closes.ix[length + i + 1] - closes.ix[length + i]
        t_i = times.ix[length + i + 1]
        a_i, b_i = generator.send(diff)
        rsi = _rsi_formula(a_i, b_i)
        results.append(_create_dict(t_i, rsi))
    return {'success': 1, 'return': {'RSIs': results}}




from json import dumps


def save_trade_log(filename, time, action, price, amount, limit=None):
    trade_log = {
        'time': time,
        'action': action,
        'price': price,
        'amount': amount,
        'limit': limit
    }
    f = open(filename, 'a')
    trade_str = dumps(trade_log)
    f.write(trade_str + '\n')
    f.close()



import time
from zaifbot.bot_common.bot_const import PERIOD_SECS, LIMIT_COUNT, LIMIT_LENGTH, UTC_JP_DIFF
from zaifbot.modules.moving_average import TradeLogsSetUp, MovingAverageSetUp
from zaifbot.modules.dao.moving_average import MovingAverageDao


def get_sma(currency_pair='btc_jpy', period='1d', count=LIMIT_COUNT,
            to_epoch_time=int(time.time()), length=LIMIT_LENGTH):
    return _get_moving_average(currency_pair, period, count, to_epoch_time, length, 'sma')


def get_ema(currency_pair='btc_jpy', period='1d', count=LIMIT_COUNT,
            to_epoch_time=int(time.time()), length=LIMIT_LENGTH):
    return _get_moving_average(currency_pair, period, count, to_epoch_time, length, 'ema')


def _get_moving_average(currency_pair, period, count, to_epoch_time, length, sma_ema):
    count = min(count, LIMIT_COUNT)
    length = min(length, LIMIT_LENGTH)
    end_time = get_end_time(to_epoch_time, period)
    tl_start_time = end_time - ((count + length) * PERIOD_SECS[period])
    ma_start_time = end_time - (count * PERIOD_SECS[period])

    trade_logs = TradeLogsSetUp(currency_pair, period, count, length)
    if trade_logs.execute(tl_start_time, end_time) is False:
        return {'success': 0, 'error': 'failed to set up trade log'}

    moving_average = MovingAverageSetUp(currency_pair, period, count, length)
    ma_result = moving_average.execute(ma_start_time, tl_start_time, end_time)
    if ma_result is False:
        return {'success': 0, 'error': 'failed to set up moving average'}
    return _create_return_dict(sma_ema, currency_pair, period,
                               length, end_time, tl_start_time, count)


def get_end_time(to_epoch_time, period):
    if PERIOD_SECS[period] > PERIOD_SECS['1h']:
        end_time = to_epoch_time - ((to_epoch_time + UTC_JP_DIFF) % PERIOD_SECS[period])
    else:
        end_time = to_epoch_time - (to_epoch_time % PERIOD_SECS[period])
    return end_time


def _create_return_dict(sma_ema, currency_pair, period, length, end_time, tl_start_time, count):
    return_datas = []
    moving_average = MovingAverageDao(currency_pair, period, length)
    ma_result = moving_average.get_trade_logs_moving_average(end_time, tl_start_time)
    if len(ma_result) < count:
        return {'success': 0, 'error': 'moving average data is missing'}
    for i in ma_result:
        if sma_ema == 'sma' and i.MovingAverages:
            moving_average = i.MovingAverages.sma
        elif sma_ema == 'ema' and i.MovingAverages:
            moving_average = i.MovingAverages.ema
        else:
            moving_average = 0.0
        return_datas.append({'time_stamp': i.TradeLogs.time, 'moving_average': moving_average,
                             'close': i.TradeLogs.close, 'closed': bool(i.TradeLogs.closed)})
    return {'success': 1, 'return': {sma_ema: return_datas}}



from sqlalchemy import Column, Integer, Float, String

from zaifbot.models import Base


class BollingerBands(Base):
    __tablename__ = 'bollinger_bands'
    time = Column('time', Integer, primary_key=True)
    currency_pair = Column('currency_pair', String, primary_key=True)
    period = Column('period', String, primary_key=True)
    length = Column('length', Integer, primary_key=True)
    sd1p = Column('sd1p', Float, nullable=False)
    sd2p = Column('sd2p', Float, nullable=False)
    sd3p = Column('sd3p', Float, nullable=False)
    sd1n = Column('sd1n', Float, nullable=False)
    sd2n = Column('sd2n', Float, nullable=False)
    sd3n = Column('sd3n', Float, nullable=False)
    closed = Column('closed', Integer, nullable=False)
